/*
 * SDL port done by exelenz (ivan@solvo.ru) @ 14/10/2001.
 */

#ifndef __EGL_H
#define __EGL_H

/****************************************************************************\
* $Header: /C/ElSocc/EGL_DD/EGL_DD.h 6     14.03.99 12:21 Yole $
*  *
* File 'egl.h'  - Definitions for EGL                                        *
* Last modified - 22:11:43, 20.09.97                                         *
*                                                                            *
* Written by EXeLENZ (2:5030/448.1@fidonet, ivan@solvo.ru)                   *
* Changes and additions by Yoletir (55:1055/246@general, 2:5030/654@fidonet) *
*  *
\****************************************************************************/

#include "egl_defs.h"

#ifdef _WIN32
#include <windows.h>
#endif

typedef unsigned char * VBuffer;      // Type - Video Buffer
typedef char  Bool;                   // Type - boolean expression
typedef int   EGLcolor;               // color of a pixel

extern char    EGLbytesPerPixel;      // bits per pixel for current mode
extern VBuffer EGLvBuffer;            // current video buffer 

struct BitMap {                       // BitMap - image holds size and
   ushort XSize;                      // pointer to image
   ushort YSize;
   uchar* Image;
};

////
// initialisation/service routines                                          //
////

/****************************************************************************\
* Function : EGLhaveMode                                                     *
* Desc     : Determines if particular video mode is supported                *
* Input    : resolution, bpp                                                 *
* Output   : success/failure                                                 *
****************************************************************************/
int EGLhaveMode (int x, int y, int bpp);

/****************************************************************************\
* Function : EGLinitSDL                                                      *
* Desc     : Sets needed video mode                                          *
* Input    : x,y, bpp, if it is fullscreen. rest is obsolete                 *
* Output   : success/failure                                                 *
\****************************************************************************/
int EGLinitSDL (int x, int y, int bpp, int bFullscreen,
                int BufX, int BufY, int bNoStretch);

/****************************************************************************\
* Function : EGLterminate                                                    *
* Desc     : Terminate EGL                                                   *
* Input    : none                                                            *
* Output   : none                                                            *
\****************************************************************************/
void EGLterminate (void);

////
// Virtual buffer                                                           //
////

/****************************************************************************\
* Function : EGLcreateVBuffer                                                *
* Desc     : Creates virtual bufer                                           *
* Input    : pointer to v buffer                                             *
* Output   : success/failure                                                 *
\****************************************************************************/
int EGLcreateVBuffer (VBuffer * buffer);

/****************************************************************************\
* Function : EGLsetActiveVBuffer                                             *
* Desc     : Set active virtual buffer                                       *
* Input    : buffer                                                          *
* Output   : success/failure                                                 *
\****************************************************************************/
int EGLsetActiveVBuffer (VBuffer buffer);

/****************************************************************************\
* Function : EGLclearVBuffer                                                 *
* Desc     : Clear buffer                                                    *
* Input    : buffer                                                          *
* Output   : success/failure                                                 *
\****************************************************************************/
int EGLclearVBuffer (VBuffer buffer);

/****************************************************************************\
* Function : EGLdrawVBuffer                                                  *
* Desc     : Draw buffer on screen                                           *
* Input    : virtual buffer                                                  *
* Output   : ok/error                                                        *
\****************************************************************************/
int EGLdrawVBuffer (VBuffer buffer);

////
// Sprite handling                                                          //
////

/****************************************************************************\
* Function : EGLallocBlock                                                   *
* Desc     : Alllocate memory for eglBlock                                   *
* Input    : BitMap, X size of image, Y size of image                        *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLallocBlock (BitMap ** block,
                   short xsize,
                   short ysize);

/****************************************************************************\
* Function : EGLfreeBlock                                                    *
* Desc     : Free sprite                                                     *
* Input    : BitMap                                                          *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLfreeBlock (BitMap * block);

/****************************************************************************\
* Function : EGLgetBlock                                                     *
* Desc     : Stores rectangle region of VBuffer in BitMap                    *
* Input    : Destination bitmap, x ,y ,xrl ,yrl                              *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLgetBlock (BitMap* block,
                  ushort x,
                  ushort y,
                  ushort xrl,
                  ushort yrl);

/****************************************************************************\
* Function : EGLputBlock                                                     *
* Desc     : Puts rectangle region of picture in EGLvBuffer                  *
* Input    : Source bitmap, x ,y                                             *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLputBlock (BitMap* block,
                  ushort x,
                  ushort y);

/****************************************************************************\
* Function : EGLwaitVRetrace                                                 *
* Desc     : Wait for vertical retrace                                           *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void EGLwaitVRetrace (void);

/****************************************************************************\
* Function : EGLgetPixelFormat                                               *
* Desc     : Gets pixel format of the screen (R,G,B masks)                   *
* Input    : None                                                            *
* Output   : Masks                                                           *
\****************************************************************************/
int EGLgetPixelFormat (ulong *RMask, ulong *GMask, ulong *BMask);

#endif

/*
 * $Log: /C/ElSocc/EGL_DD/EGL_DD.h $
 * 
 * 6     14.03.99 12:21 Yole
 * EGLhaveMode added
 * 
 * 5     5.01.99 1:09 Yole
 * supports stretching
 * 
 * 4     4.01.99 0:56 Yole
 * windowed mode support (incomplete)
 * 
 * 3     2.01.99 13:38 Yole
 * fixed EGLgetBlock/EGLputBlock
 * 
 * 2     31.12.98 1:31 Yole
 * work in progress
 * 
 * 1     26.12.98 0:48 Yole
 * Quick & dirty port of EGL under DirectDraw
*/