/****************************************************************************\
*  *
* File 'el_sdl.cpp'   - Electron Soccer v 1.2, SDL port                      *
* Project started     - 14:54:21, 18-11-97                                   *
* Last modified       - 06:11:28, 30-11-01                                   *
*                                                                            *
* Written by EXeLENZ (2:5030/448.1@fidonet, ivan@solvo.ru)                   *
*  *
\****************************************************************************/

#include <stdarg.h>
#include <stdio.h>

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#endif

#include "el_socc.h"
#include "el_music.h"
#include "fxres.h"

// Ini parser
#include "./ini_parser/src/e_error.h"
#include "./ini_parser/src/iniparser.h"

#include "SDL.h"

int EGLinitSDL (int x, int y, int bpp, int bFullscreen,
                int BufX, int BufY, int bNoStretch);

int bActive;                             /* Whether game is paused or not   */
int bFullscreen=1;                       /* Whether we are in full screen   */
int bNoStretch=0;
int VidMaxX, VidMaxY;

extern int RedKeys   [4];
extern int GreenKeys [4];

int main( int argc, char* argv[] ) 
{

   if (!ResInitFile ("els.res")) {

#ifdef _WIN32
	  MessageBox (NULL, "Cannot open resource file", "Electron Soccer", MB_OK);
#else
	  e_error ("Cannot open resource file!");
#endif
      return FALSE;
   }

   bNoStretch=1;
   VidMaxX = 320; VidMaxY = 200;

   // Read config
   dictionary	*	ini ;
   ini = iniparser_load("elsocc.ini");

#ifdef DEBUG
   FILE * t;
   t=fopen ("dump.ini", "wb");
   iniparser_dump(ini, t);
   fclose (t);
#endif

   if (ini==NULL) {
#ifdef _WIN32
	  MessageBox (NULL, "Cannot open ini file [elsocc.ini], loading defaults!", "Electron Soccer", MB_OK);
#else
	  e_error("Cannot open ini file [elsocc.ini], loading defaults!");
#endif

   } else {
	  if(iniparser_getint(ini, "general:enableplasma", -1)==0) NOPLASMA=YES;
	  if(iniparser_getint(ini, "audio:enablesound", -1)==0) NOSOUND=YES;
	  if(iniparser_getint(ini, "audio:enablemusic", -1)==0) NOMUSIC=YES;

	  if(iniparser_getint(ini, "general:gamespeed", -1)!=-1) {
	     GameSpeed = ((float) iniparser_getint(ini, "general:gamespeed", -1))/10;
	  }

	  if(iniparser_getint(ini, "general:gamelength", -1)!=-1) {
          GameTime = iniparser_getint(ini, "general:gamelength", -1);
	  }

      if (iniparser_getint(ini, "video:windowed", -1)==1) bFullscreen=0;
 
      // Load controls

      if(iniparser_getint(ini, "controls/redplayer:left", -1)!=-1) 
         RedKeys[0]=iniparser_getint(ini, "controls/redplayer:left", -1); 
      if(iniparser_getint(ini, "controls/redplayer:right", -1)!=-1) 
         RedKeys[1]=iniparser_getint(ini, "controls/redplayer:right", -1); 
      if(iniparser_getint(ini, "controls/redplayer:up", -1)!=-1) 
         RedKeys[2]=iniparser_getint(ini, "controls/redplayer:up", -1); 
      if(iniparser_getint(ini, "controls/redplayer:down", -1)!=-1) 
         RedKeys[3]=iniparser_getint(ini, "controls/redplayer:down", -1); 

      if(iniparser_getint(ini, "controls/greenplayer:left", -1)!=-1) 
         GreenKeys[0]=iniparser_getint(ini, "controls/greenplayer:left", -1); 
      if(iniparser_getint(ini, "controls/greenplayer:right", -1)!=-1) 
         GreenKeys[1]=iniparser_getint(ini, "controls/greenplayer:right", -1); 
      if(iniparser_getint(ini, "controls/greenplayer:up", -1)!=-1) 
         GreenKeys[2]=iniparser_getint(ini, "controls/greenplayer:up", -1); 
      if(iniparser_getint(ini, "controls/greenplayer:down", -1)!=-1) 
         GreenKeys[3]=iniparser_getint(ini, "controls/greenplayer:down", -1); 
   }

   iniparser_freedict(ini);

   EGLinitSDL (320,200,15,bFullscreen,1,1,1);

   ElInitMusic();

   ElSoccMain();

   return (1);
}


void delay (unsigned int __milliseconds)
{
   SDL_Delay (__milliseconds);
}

void DeinitKbd()
{
}

/****************************************************************************\
* Function : Error                                                           *
* Desc     : Finishes application, writes error message                      *
* Input    : reason                                                          *
* Output   : none                                                            *
\****************************************************************************/
void Error (int cause) {

   EGLterminate ();
   switch (cause) {
   case ErrMusax :
#ifdef _WIN23
	  MessageBox (NULL, "Trouble with audio engine, sorry.", "Electron Soccer", MB_OK);
#endif
      break;
   }

   exit (cause);
}

/****************************************************************************\
* Function : EGLtrace                                                        *
* Desc     : Trace                                                           *
* Input    : format, args                                                    *
* Output   : none                                                            *
\****************************************************************************/
void /*__cdecl*/ EGLtrace (char *fmt, ...)
{
   char buf [256];
   va_list va;

   va_start (va, fmt);
   vsprintf (buf, fmt, va);
   va_end (va);

   // MessageBox (NULL, buf, "Electron Soccer", MB_OK);
#ifdef _WIN32   
   OutputDebugString (buf);
#endif

}

/****************************************************************************\
* Function : ProcessMessages                                                 *
* Desc     : Processes SDL keyboard events                                   *
\****************************************************************************/
void ProcessMessages() {
   int ScanCode;
   SDL_Event event;

   /* Poll for events. SDL_PollEvent() returns 0 when there are no  */
   /* more events on the event queue, our while loop will exit when */
   /* that occurs.                                                  */
   while( SDL_PollEvent( &event ) ){
      /* We are only worried about SDL_KEYDOWN and SDL_KEYUP events */
      switch( event.type ){
         case SDL_KEYDOWN:
            ScanCode = event.key.keysym.sym % 512;
            EGLtrace ("SDL key code :%d\n", ScanCode);
            keymap [ScanCode] = PRESSED;
            break;

         case SDL_KEYUP:
            ScanCode = event.key.keysym.sym % 512;
            keymap [ScanCode] = DEPRESSED;
            break;

         default:
            break;
      }
   }
}
