/****************************************************************************\
*  *
* File 'el_socc.cpp'  - Electron Soccer v 1.2                                *
* Project started     - 14:54:21, 18-11-97                                   *
* Last modified       - 22:11:28, 29-11-01                                   *
*                                                                            *
* Written by EXeLENZ (2:5030/448.1@fidonet, ivan@solvo.ru)                   *
* Win32 version written by Yoletir / FaerieFire AKA Dmitry Jemerov,          *
*                                                              2:5030/654    *
*  *
\****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "els.h"
#include "el_socc.h"
#include "el_music.h"

#include "fxres.h"

#include "SDL.h"

#ifdef USE_FMOD

static FMUSIC_MODULE *module;

#endif

int RedKeys   [4] = {SDLK_LEFT, SDLK_RIGHT, SDLK_UP, SDLK_DOWN};
int GreenKeys [4] = {SDLK_o, SDLK_p, SDLK_q, SDLK_a};

char keymap [512];                     /* Keyboard status                   */

VBuffer screen1;                       /* EGL virtual frame buffer          */

int isQueue=NO;                        /* If queue was created?             */

struct Queue {                         /* Object queue                      */
   Qelem * first;                      /* First object                      */
   Qelem * last;                       /* Last object                       */
   int len;                            /* Queue length                      */
} queue;

Qelem TempElem;

BitMap * r_ball;                         /* r  - ball                       */
BitMap * g_ball;                         /* g  - ball                      */
BitMap * w_ball;                         /* w  - ball                      */
BitMap * rs_ball;                        /* rs - ball                      */
BitMap * gs_ball;                        /* gs - ball                      */
BitMap * ws_ball;                        /* ws - ball                      */
BitMap * ws_move;                        /* ws - ball                      */

BitMap * b_dummy;                        /* Temporary bitmap                */

BitMap * r_bgr;                          /* Background 4 red player score   */
BitMap * g_bgr;                          /* Background 4 green player score */
BitMap * time_bgr;                       /* Background for time             */

BitMap * logo_bmp;
BitMap * cpic_bmp;
BitMap * playfield;
BitMap * goal_r_bmp;
BitMap * goal_g_bmp;
BitMap * gr_d;
BitMap * gr_u;
BitMap * gr_quest;
BitMap * gr_ok;
BitMap * time_bmp;
BitMap * ws_move_bmp;
BitMap * diss_g_bmp;
BitMap * diss_r_bmp;
BitMap * yea_bmp;
BitMap * red_wins_bmp;
BitMap * green_wins_bmp;
BitMap * win_bmp;
BitMap * oops;
BitMap * time_over;

int WannaQuit=NO;

float transA, transB;                    /* Transparency                    */
float corr=0;

int AlpLim=15;                           /* Transparency threshold          */

float MaxSp=3.5;                         /* Maximal speed in the game       */

int scoreRed;                            /* Red player score                */
int scoreGreen;                          /* Green player score              */

long int frames;                         /* Frame counter (since start)     */

int OneHitDone=NO;

uchar SinTable [256], CosTable [256];    /* Sines and cosines               */

VBuffer plasma256;                       /* Indexed plasma                  */
ushort Ctab [256];                       /* Indexed -> HC conversion table  */
char i3,j3;
int end_y;
double a1=1,a2=0,a3=0;                   /* Plasma colors                   */
double add=-0.01,add2=0;

int NOPLASMA=NO;                         /* Disables plasma                 */
int NOSOUND=NO;                          /* Disables sound/music            */
int NOMUSIC=NO;                          /* Disables music only             */

char collis [100] [100];                 /* Collision table                 */

volatile int inHelp=NO,inRed=NO;         /* Flag - is in help/red keys      */
volatile int inInter=NO;

static ushort RMask=0, GMask=0, BMask=0,
              RMaskShifted=0, GMaskShifted=0, BMaskShifted=0,
              RShift=0, GShift=0, BShift=0, GBits=0;

double GameSpeed=1;                      /* Game speed                      */
int GameTime=84;                         /* Game length in seconds          */

#define M_PI  3.14159265358

/* DEBUG */

#ifdef _DEBUG

void _EGLassert (char *expr, char *file, int line)
{
   EGLterminate();
   printf ("Assertion failed: %s\n@ file %s, line %i\n", expr, file, line);
   EGLtrace ("Assertion failed: %s\n@ file %s, line %i\n", expr, file, line);
   exit (1);
}


#define EGLassert(expr) ((expr)?(void)0:_EGLassert(#expr,__FILE__,__LINE__))
#define EGLcheck(expr) ((expr)?(void)0:_EGLassert(#expr,__FILE__,__LINE__))

#else

#define EGLassert(__ignore) ((void)0)
#define EGLcheck(expr) (expr)

#endif

inline ushort ElsHC (ushort r, ushort g, ushort b) {
   return ((r << RShift) & RMask) |
      ((g << GShift) & GMask) |
      ((b << BShift) & BMask);
}

void ElSoccMain()
{
   // Definitions for plasma.
   plasma256=(uchar *) malloc (PlasmaX*PlasmaY*2+10);
   memset (plasma256,0,PlasmaX*PlasmaY*2);

   Demix (ID_README, "readme");
   Demix (ID_DIZ, "file_id.diz");

   ElLoadMusic();

   // Calculate pixel format (rgb)
   UpdatePixelFormat();

   // Set active virtual buffer
   screen1=(uchar *) malloc (MaxX*MaxY*2);
   memset (screen1,0,MaxX*MaxY*2);
   EGLsetActiveVBuffer (screen1);

   logo_bmp=EGLloadFFI (ID_LOGO);
   cpic_bmp=EGLloadFFI (ID_CPIC);

   gr_u=EGLloadFFI (ID_GR_UP);
   gr_d=EGLloadFFI (ID_GR_DOWN);

   EGLallocBlock (&r_bgr,150,10);
   EGLallocBlock (&g_bgr,150,10);
   EGLallocBlock (&time_bgr,150,10);

   // EGLblock for rotating half'o'ball.
   ws_move_bmp=EGLloadFFI (ID_WS_MOVE);
   EGLallocBlock (&ws_move, 11, 9);
   ws_move->Image=ws_move_bmp->Image;
   ws_move->XSize = 10;
   ws_move->YSize = 8;

   r_ball=EGLloadFFI (ID_R_BALL);
   g_ball=EGLloadFFI (ID_G_BALL);
   w_ball=EGLloadFFI (ID_W_BALL);

   playfield=EGLloadFFI (ID_PLAYFIELD);

   goal_r_bmp=EGLloadFFI (ID_GOAL_RED);
   goal_g_bmp=EGLloadFFI (ID_GOAL_GREEN);

   time_bmp=EGLloadFFI (ID_TIME);

   diss_r_bmp=EGLloadFFI (ID_DISS_R);
   diss_g_bmp=EGLloadFFI (ID_DISS_G);

   yea_bmp=EGLloadFFI (ID_YEA);
   green_wins_bmp=EGLloadFFI (ID_GREEN_WINS);
   red_wins_bmp=EGLloadFFI (ID_RED_WINS);
   win_bmp=EGLloadFFI (ID_WIN);

   oops=EGLloadFFI (ID_OOPS);
   time_over=EGLloadFFI (ID_TIME_OVER);

   // Generate color table for indexed->hc conversion.
   for (int z=0;z<64;z++) {
      Ctab[z]     = ElsHC (z/6,z/2,z/6);
      Ctab[127-z] = ElsHC (z/6,z/2,z/6);
      Ctab[127+z] = ElsHC (z/6,z/2,z/6);
      Ctab[254-z] = ElsHC (z/6,z/2,z/6);
   }

   // Sin and Cos tables.
   for (int i=0; i<256; i++) {
     SinTable [i]=(uchar) floor (sin (2*M_PI*i/255)*128+128);
     CosTable [i]=(uchar) floor (cos (2*M_PI*i/255)*128+128);
   }

   i3=10; j3=90;

   // Yo ! Lets start !
   Game ();
}

void UpdatePixelFormat()
{
   ulong RMaskTemp, GMaskTemp, BMaskTemp;
   EGLgetPixelFormat (&RMaskTemp, &GMaskTemp, &BMaskTemp);

   RMask = (ushort) RMaskTemp;
   GMask = (ushort) GMaskTemp;
   BMask = (ushort) BMaskTemp;

   EGLtrace ("RMask %04X, BMask %04X, GMask %04X\n", RMask, GMask, BMask);

   for (RShift=0, RMaskShifted=RMask;
        (RMaskShifted & 1) == 0;
        RMaskShifted >>= 1)
      RShift++;

   for (GShift=0, GMaskShifted=GMask;
        (GMaskShifted & 1) == 0;
        GMaskShifted >>= 1)
      GShift++;

   for (BShift=0, BMaskShifted=BMask;
        (BMaskShifted & 1) == 0;
        BMaskShifted >>= 1)
      BShift++;

   for (int bit=0; bit<16; bit++)
      if ((GMask & (1 << bit)) != 0)
         GBits++;
}

void ElSoccDone()
{
   ElShutdownMusic();

   DestroyQueue();

   free (plasma256);

   ResClose();

   EGLterminate ();
   exit (1);
}

/****************************************************************************\
* Function : Game                                                            *
* Desc     : Game cycle in general                                           *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void Game (void) {
   EGLtrace ("Game!\n");

   for (int wer=0;wer<512;wer++) keymap[wer]=0;

   while (1) {
      Intro();

      // Draw playfield         
      EGLputBlock (playfield, 0, 0);

      // Draw static background              
      EGLgetBlock (r_bgr,70,0,130,10);
      EGLgetBlock (g_bgr,70,190,130,200);
      EGLgetBlock (time_bgr,215,0,319,10);

      int Goal=0;
      scoreRed=0;
      scoreGreen=0;
      int timeLeft=1000*GameTime;
      int timeNextUpdate=timeLeft;
      frames=0;
      WannaQuit=NO;

      // Initialise object queue              
      InitObjQueue ();

      // Adding red, green and white balls
      AddObjects ();

      // Initialise timer
      long OldFrameTime=SDL_GetTicks(), FrameTime, FrameElapsed;

      while (!WannaQuit && scoreRed <= 5 && scoreGreen <= 5) {

         //  ElSocc ⨢,    横  ProcessMessages.
         // ⮬  ६  㦭 뢠  । ⥫쭮
         // .

         FrameTime=SDL_GetTicks();

         ProcessMessages();

         // Handle keystrokes
         KbdHandler ();

         // Calculate frame time elapsed
         FrameElapsed=(FrameTime-OldFrameTime);  // milliseconds
         // EGLtrace ("frame elapsed %i usec\n", FrameTime-OldFrameTime);
         OldFrameTime=SDL_GetTicks();

         // Handle object queue
         Goal=ProcessQueue (FrameElapsed);

         if (timeLeft > 0) {
            timeLeft -= FrameElapsed;
            if (timeLeft < 0)  {
               timeOver();
               OldFrameTime=SDL_GetTicks();
            }
            else if (timeLeft < timeNextUpdate) {
               WriteTime (timeLeft / (1000*GameTime/84));
               timeNextUpdate -= 1000*GameTime/84;
            }
         }

         EGLdrawVBuffer (screen1);
         frames++;

         if (Goal) {
            if (Goal == Lower) scoreRed++;
            else scoreGreen++;

            for (Qelem *curElem=queue.last; curElem; curElem=curElem->prev) {

               EGLputBlock (curElem->bgr,(int) curElem->ox,(int) curElem->oy);

               curElem->ox=curElem->x;
               curElem->oy=curElem->y;
            }

#ifdef DISSOLVE_BALLS
            dissolveBalls ();
#endif

            DestroyQueue ();

            if (Goal == Lower)
               scaleOne (goal_r_bmp,100,150,1);
            else
               scaleOne (goal_g_bmp,100,50,1);

            delay (1500);

            // Initialise object queue              
            InitObjQueue ();

            // Adding red, green and white balls
            AddObjects ();

            // Update scores
            WriteScores ();

            // When moving balls, we should exclude time needed to 
            // draw goal anim
            OldFrameTime=SDL_GetTicks();
         }
      }

      if (!WannaQuit) {
         if (scoreRed > 5 && Goal == Lower) redWin();
         else if (scoreGreen > 5 && Goal == Upper) greenWin();
      }
      WaitRelease();
   }
}

//  //
// Keyboard handling                                                        //
//  //

/****************************************************************************\
* Function : KbdHandler                                                      *
* Desc     : Handles all controls                                            *
* Input    : Object                                                          *
* Output   : None                                                            *
\****************************************************************************/
void KbdHandler (void) {

   // Quitter =)
   if (keymap[SDLK_ESCAPE]) WannaQuit=YES;

   HRhan ();

   for (Qelem * curElem=queue.first; curElem; curElem=curElem->next)
      ControlObject (curElem);

   // Change alpha threshold in real time
   if (keymap[SDLK_F11]) {AlpLim++;printf ("alpha threshold set to %d",AlpLim);}
   if (keymap[SDLK_F12]) {AlpLim--;printf ("alpha threshold set to %d",AlpLim);}

//   if (keymap[44]) {corr+=0.05;}
//   if (keymap[45]) {corr-=0.05;}
}

/****************************************************************************\
* Function : Getch                                                           *
* Desc     : Waits for any key pressed/depressed                             *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void Getch (void) {
   char init_keymap [512];
   memcpy (init_keymap, keymap, 512);
   delay (10);

   do {
      ProcessMessages();
   } while (memcmp (init_keymap, keymap, 512) == 0);
}

/****************************************************************************\
* Function : KeySum                                                          *
* Desc     : Returns the sum of all key flags in the key map                 *
* Input    : None                                                            *
* Output   : Flag sum                                                        *
\****************************************************************************/

int KeySum() {
   int sum=0;
   for (int i=0; i<512; i++)
      sum += keymap [i];
   return sum;
}

/****************************************************************************\
* Function : WaitRelease                                                     *
* Desc     : Waits till all keys are released                                *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void WaitRelease (void) {

   do {
      ProcessMessages();
   } while (KeySum() != 0);
}

/****************************************************************************\
* Function : HRhan                                                           *
* Desc     : Handles help and redefine                                       *
* Input    : Object                                                          *
* Output   : None                                                            *
\****************************************************************************/
void HRhan (void) {
   // On line help =)
   if ((keymap[SDLK_F1])&&(inHelp==NO)) {
      inHelp=YES;
      GrHelp ();
      inHelp=NO;
   }

   // Redefine keys
   if ((keymap[SDLK_F2])&&(inRed==NO)) {
      RedefineKeys ();
   }

   return;
}

/****************************************************************************\
* Function : ControlObject                                                   *
* Desc     : Handles controls for one object                                 *
* Input    : Object                                                          *
* Output   : None                                                            *
\****************************************************************************/
void ControlObject (Qelem * elem)
{
   if (keymap[elem->left])  elem->xspeed-=0.1;
   if (keymap[elem->right]) elem->xspeed+=0.1;
   if (keymap[elem->up])    elem->yspeed-=0.1;
   if (keymap[elem->down])  elem->yspeed+=0.1;

   if (elem->xspeed>MaxSp) elem->xspeed=MaxSp;
   if (elem->xspeed<-MaxSp) elem->xspeed=-MaxSp;

   if (elem->yspeed>MaxSp) elem->yspeed=MaxSp;
   if (elem->yspeed<-MaxSp) elem->yspeed=-MaxSp;
}


//  //
// Queue functions                                                          //
//  //

/****************************************************************************\
* Function : RestoreBackrounds                                               *
* Desc     : Restores backgrounds of all objects in the queue                *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/

void RestoreBackgrounds()
{
   for (Qelem *curElem=queue.last; curElem; curElem=curElem->prev) {

      EGLputBlock (curElem->bgr,(int) curElem->ox,(int) curElem->oy);

      curElem->ox=curElem->x;
      curElem->oy=curElem->y;
   }
}

/****************************************************************************\
* Function : ProcessQueue                                                    *
* Desc     : Manages all objects in queue (moves them, etz)                  *
* Input    : None                                                            *
* Output   : Goal status                                                     *
\****************************************************************************/
int ProcessQueue (long FrameElapsed)
{
   RestoreBackgrounds();

   if (frames%10==0) {
      a1=a1+add;
      a2=a2-add;
      a3=1.3-a2-a1;
      if (a3>1) a3=1;
      if (a3<0) a3=0;
      if (a1>0.8) add=-0.01;
      if (a1<0.2) add=+0.01;
      // Cenerate color table for indexed->hc conversion.
      for (int z=0;z<64;z++) {
         ushort c = ElsHC ((int)((float)z/2*a1),
                           (int)((float)(z >> (6-GBits))*a2),
                           (int)((float)z/2*a3));
         Ctab[z] = c;
         Ctab[127-z] = c;
         Ctab[127+z] = c;
         Ctab[254-z] = c;
       }
   }


   // Redraw bacground plasma every 2 frames.
   if (frames%2==0&&NOPLASMA==NO) {
      DrawPlasma ();
      ConvertPlasma ();
   }

   int Goal=0;

   memset (collis,0,sizeof (collis));

   for (Qelem *curElem=queue.first; curElem; curElem=curElem->next) {

      /*EGLtrace ("elem at (%f,%f), speed (%f,%f)\n",
                curElem->x, curElem->y, curElem->xspeed, curElem->yspeed); */

      EGLassert (curElem->bgr != NULL);

      // Store new background
      EGLgetBlock (curElem->bgr,
                   (int)curElem->x,
                   (int)curElem->y,
                   (int)curElem->x+curElem->image->XSize,
                   (int)curElem->y+curElem->image->YSize);

      // Draw object on new place
      EGLputBlockT (curElem->image,(int)curElem->x,(int)curElem->y,10);

      // Handle wall bouncing
      BounceWalls (curElem,
                   curElem->xspeed*FrameElapsed*GameSpeed/10,  // /10 - ⮡  ᪮ ࠢ﫠 1);
                   curElem->yspeed*FrameElapsed*GameSpeed/10);

      /*EGLtrace ("moved to (%f,%f)\n", curElem->x, curElem->y);*/

      // Handle object interaction
      if (!OneHitDone) {
         BounceObjects (curElem);
      } else OneHitDone=NO;

      // Check if this is a goal
      int CurGoal=CheckGoal (curElem);
      if (CurGoal) Goal=CurGoal;
   }

   // Draw scores
   if (frames%7==0) WriteScores ();

   return Goal;
}

/****************************************************************************\
* Function : InitObjQueue                                                    *
* Desc     : Initialises object queue.                                       *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void InitObjQueue (void) {

   isQueue=YES;
   queue.first=NULL;
   queue.last=NULL;
   queue.len=0;

}

/****************************************************************************\
* Function : DestroyQueue                                                    *
* Desc     : Destroys queue                                                  *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void DestroyQueue (void) {

   Qelem * curElem;
   Qelem * delElem;

   curElem = queue.first;

   if (queue.first == NULL) return;

   do {
      delElem = curElem;
      curElem = curElem->next;
      EGLfreeBlock (delElem->bgr);
      free (delElem);

   } while (curElem->next != NULL);

   queue.first=NULL;
   queue.last=NULL;
   queue.len=0;

   isQueue=NO;
}

/****************************************************************************\
* Function : AddObjToQueue                                                   *
* Desc     : Adds single object to queue                                     *
* Input    : Object to add                                                   *
* Output   : None                                                            *
\****************************************************************************/
void AddObjToQueue (Qelem TempElem) {

   Qelem * newElem;
   Qelem * curElem;

   newElem = (Qelem *) malloc (sizeof (Qelem));

   if (queue.first == NULL) {
      queue.first=newElem;
      newElem->prev=NULL;
      newElem->next=NULL;
   }
   else {
      curElem=queue.last;
      curElem->next=newElem;
      newElem->prev=curElem;

   }

   queue.len++;
   queue.last=newElem;

   newElem->bgr          = (BitMap *) malloc (sizeof (BitMap));
   newElem->bgr->XSize   = TempElem.image->XSize;
   newElem->bgr->YSize   = TempElem.image->YSize;
   newElem->bgr->Image   = (uchar *)
      malloc ((TempElem.image->XSize+1)*(TempElem.image->YSize+1)*2);

   newElem->Id        = queue.len;
   newElem->image     = TempElem.image;
   newElem->speed     = TempElem.speed;
   newElem->xspeed    = TempElem.xspeed;
   newElem->yspeed    = TempElem.yspeed;
   newElem->x         = TempElem.x;
   newElem->y         = TempElem.y;
   newElem->ox        = TempElem.x;
   newElem->oy        = TempElem.y;
   newElem->weight    = TempElem.weight;
   newElem->left      = TempElem.left;
   newElem->right     = TempElem.right;
   newElem->up        = TempElem.up;
   newElem->down      = TempElem.down;
   newElem->mode      = TempElem.mode;

   newElem->LastCollide = NULL;
   newElem->next      = NULL;

   // We assume here that background is already set
   EGLgetBlock (newElem->bgr,
          (int)(newElem)->x,
          (int)(newElem)->y,
          (int)(newElem)->x+newElem->bgr->XSize,
          (int)(newElem)->y+newElem->bgr->YSize);
}


//  //
// Collision / game conditions handling                                     //
//  //

/****************************************************************************\
* Function : BounceWalls                                                     *
* Desc     : Handles collisions with walls                                   *
* Input    : Object to handle, X and Y movement distances                    *
* Output   : None                                                            *
\****************************************************************************/
void BounceWalls (Qelem * curElem, double move_x, double move_y)
{
   char bounced=NO;

   /* bounce lower wall */
   if (curElem->y + move_y > (183-curElem->image->YSize)) {

      /* white ball can go through gates and bounce from their borders */
      if (curElem->mode == hitAll &&
          curElem->x >= VorX &&
          curElem->x+curElem->image->XSize <= VorX+VorSize) {
         /* in gates we need to determine if horisontal bounce is needed */
         if (curElem->x+move_x < VorX) {
            curElem->x = 2*VorX-curElem->x-move_x;
            curElem->xspeed = -curElem->xspeed;
            move_x=0;
            bounced=YES;
         }
         else if (curElem->x+move_x > VorX+VorSize-curElem->image->XSize) {
            curElem->x = 2*(VorX+VorSize-curElem->image->XSize)-curElem->x-move_x;
            curElem->xspeed = -curElem->xspeed;
            move_x=0;
            bounced=YES;
         }
         curElem->y += move_y;
      }
      /* if not in gates and bal is not white, then use basic bouncing */
      else {
         curElem->y = 2*(183-curElem->image->YSize) - curElem->y - move_y;
         curElem->yspeed=-curElem->yspeed;
         bounced=YES;
      }
   }

   /* bounce from upper wall */
   else if (curElem->y + move_y < 16) {
      /* white ball can go through gates and bounce from their borders */
      if (curElem->mode == hitAll &&
          curElem->x >= VorX &&
          curElem->x+curElem->image->XSize <= VorX+VorSize) {
         /* in gates we need to determine if horisontal bounce is needed */
         if (curElem->x+move_x < VorX) {
            curElem->x = 2*VorX-curElem->x-move_x;
            curElem->xspeed = -curElem->xspeed;
            move_x=0;
            bounced=YES;
         }
         else if (curElem->x+move_x > VorX+VorSize-curElem->image->XSize) {
            curElem->x = 2*(VorX+VorSize-curElem->image->XSize)-curElem->x-move_x;
            curElem->xspeed = -curElem->xspeed;
            move_x=0;
            bounced=YES;
         }
         curElem->y += move_y;
      }
      /* if not in gates and bal is not white, then use basic bouncing */
      else {
         curElem->y = 2*16 - curElem->y - move_y;
         curElem->yspeed=-curElem->yspeed;
         bounced=YES;
      }
   }
   else curElem->y += move_y;

   /* handle right wall */
   if (curElem->x + move_x > (311-curElem->image->XSize)) {
      curElem->x = 2*(311-curElem->image->XSize) - curElem->x - move_x;
      curElem->xspeed = -curElem->xspeed;
      bounced=YES;
   }
   /* handle left wall */
   else if (curElem->x + move_x < 8) {
      curElem->x = 2*8 - curElem->x - move_x;
      curElem->xspeed = -curElem->xspeed;
      bounced=YES;
   }
   else
      curElem->x += move_x;

   // Add some randomness
   if (bounced==YES) {
      curElem->xspeed += (float)(random(3)-1)/50;
      curElem->yspeed += (float)(random(3)-1)/50;
   }

   // 㪨 ⮫.
   if (bounced==YES)
	   ElPlaySample (SAMPLE_HIT, 20000);
}

/****************************************************************************\
* Function : BounceObjects                                                   *
* Desc     : Handles collisions with objects                                 *
* Input    : Object to handle                                                *
* Output   : None                                                            *
\****************************************************************************/
void BounceObjects (Qelem * curElem)
{
   int ourX,ourY;
   int ourXsize,ourYsize;
   double tempXsp,tempYsp;
   int collided=0;

   ourX = (int) curElem->x;
   ourY = (int) curElem->y;
   ourXsize = (curElem->image)->XSize;
   ourYsize = (curElem->image)->YSize;

   Qelem * elem;

   elem=queue.first;

   for (int i=0;i<queue.len-1;i++) {
      if ((ourX+ourXsize-2>elem->x)&&
          (ourX+2<elem->x+(elem->image)->XSize)&&
          (ourY+ourYsize-2>elem->y)&&
          (ourY+2<elem->y+(elem->image)->YSize)&&
          (!collis [elem->Id] [curElem->Id])&&
          (elem->mode==hitAll||
          (elem->mode==hitMain&&curElem->mode==hitAll))) {

         collided=1;
         if (elem == curElem->LastCollide) {
            elem=elem->next;
            continue;
         }
         curElem->LastCollide = elem;

         tempXsp=curElem->xspeed;
         tempYsp=curElem->yspeed;

         curElem->xspeed=elem->xspeed;
         curElem->yspeed=elem->yspeed;

         elem->xspeed=tempXsp;
         elem->yspeed=tempYsp;

         // Bounce from single object only once
         collis [elem->Id][curElem->Id]=1;

		 ElPlaySample (SAMPLE_HIT1, 24000);

         OneHitDone=YES;
         return;
      }

      elem=elem->next;
   }

   if (!collided) curElem->LastCollide=NULL;
}

/****************************************************************************\
* Function : CheckGoal                                                       *
* Desc     : Checks for winning conditions                                   *
* Input    : Object to handle                                                *
* Output   : Goal status                                                     *
\****************************************************************************/
int CheckGoal (Qelem * curElem) {

   if (curElem->mode!=hitAll) return 0;

   if (curElem->y<0) return Upper;
   else if (curElem->y+curElem->image->YSize > GameMaxY) return Lower;
   else return 0;
}


//  //
// Miscellaneous functions                                                  //
//  //

/****************************************************************************\
* Function : WriteScores                                                     *
* Desc     : Draws goals for both sides                                      *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void WriteScores (void)
{
   static int b_frame=0;                    /* animation frame no */
   int b_frame2;

   int tempAlp=AlpLim;
   AlpLim=22;

   EGLputBlock (r_bgr,70,0);
   EGLputBlock (g_bgr,70,190);

   b_frame++;
   if (b_frame>29) b_frame=0;
   b_frame2=b_frame+15;
   if (b_frame2>29) b_frame2=b_frame2-30;

   for (int i=0;i<5;i++) {
      ws_move->Image=ws_move_bmp->Image+160*b_frame;
      if (scoreRed>i)   EGLputBlockT (ws_move,70+12*i,0,1);
      ws_move->Image=ws_move_bmp->Image+160*b_frame2;
      if (scoreGreen>i) EGLputBlockT (ws_move,70+12*i,191,1);
   }

   AlpLim=tempAlp;
}

/****************************************************************************\
* Function : WriteTime                                                       *
* Desc     : Draws time left                                                 *
* Input    : Time left                                                       *
* Output   : None                                                            *
\****************************************************************************/
void WriteTime (int timeLeft)
{
   int tempAlp;
   BitMap temp;
   uchar * temp_i;
   int cur=0,
       cur1=0;

   tempAlp=AlpLim;
   AlpLim=15;

   temp_i=(uchar *) malloc (2000);

   EGLputBlock (time_bgr,215,0);

   for (int j=0;j<8;j++) {

      int i;
      // Left corner
      for (i=0;i<9*2;i++) {
         temp_i[cur++]=time_bmp->Image [cur1++];
      }
      // Body
      for (int l=0;l<84;l++) {
         if (timeLeft>l) {
            temp_i[cur++]=time_bmp->Image [cur1++];
            temp_i[cur++]=time_bmp->Image [cur1++];
         } else  {
            cur1++;
            cur1++;
         }
      }
      // Right corner
      for (i=0;i<8*2;i++) {
         temp_i[cur++]=time_bmp->Image [cur1++];
      }
      // Remainder
      for (i=0;i<(84-timeLeft)*2;i++) {
         temp_i[cur++]=0;
      }

   }

   temp.Image=temp_i;
   temp.XSize = 101;
   temp.YSize = 8;

   EGLputBlockT (&temp,215,0,1);

   AlpLim=tempAlp;

   free (temp_i);
}

/****************************************************************************\
* Function : greenWin                                                        *
* Desc     : Displays greetings and restarts                                 *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void greenWin (void)
{
   scaleOne (yea_bmp,90,100);
   scaleOne (green_wins_bmp,120,130);
   scaleOne (win_bmp,150,150);

   Getch();
}

/****************************************************************************\
* Function : redWin                                                          *
* Desc     : Displays greetings and restarts                                 *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void redWin (void)
{
   scaleOne (yea_bmp,100,100);
   scaleOne (red_wins_bmp,130,130);
   scaleOne (win_bmp,160,150);

   Getch();
}

/****************************************************************************\
* Function : timeOver                                                        *
* Desc     : Displays warning and boosts scores                              *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void timeOver (void)
{
   if (scoreRed>scoreGreen) scoreRed=5;
   else if (scoreRed<scoreGreen) scoreGreen=5;
   else {
      scoreGreen=5;
      scoreRed=5;
   }

   RestoreBackgrounds();

   BitMap *bgr;
   EGLallocBlock (&bgr, MaxX, MaxY);
   EGLgetBlock (bgr, 0, 0, MaxX-1, MaxY-1);

   scaleOne (oops,80,50);
   scaleOne (time_over,100,80);

   EGLputBlock (bgr, 0, 0);
   EGLfreeBlock (bgr);

   for (Qelem *curElem=queue.first; curElem; curElem=curElem->next) {

      EGLassert (curElem->bgr != NULL);

      // Store the new background
      EGLgetBlock (curElem->bgr,
                   (int)curElem->x,
                   (int)curElem->y,
                   (int)curElem->x+curElem->image->XSize,
                   (int)curElem->y+curElem->image->YSize);

   }
}

#ifdef DISSOLVE_BALLS
/****************************************************************************\
* Function : dissolve Balls                                                  *
* Desc     : Slowly fades balls to nothing.                                  *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
//    ᠭ !  ࠡ⠥, ⠪   .
// The above comment is better left untranslated. :-)
void dissolveBalls (void)
{

   int tempAlp;                        // Temporary adjust alpha threshold
   int mov_frame=0;                    // No of frame
   BitMap temp;
   BitMap * tempbgr;
   BitMap * tempbgr1;
   Qelem * curElem;
   int x,y,x1,y1;

   tempAlp=AlpLim;
   AlpLim=22;

   //EGLallocBlock (&temp, 42, 34);

   EGLallocBlock (&tempbgr, 42, 34);
   tempbgr->XSize = 42;
   tempbgr->YSize = 34;

   EGLallocBlock (&tempbgr1, 33, 34);
   tempbgr1->XSize = 31;
   tempbgr1->YSize = 34;


   // Balls coordinates
   curElem = queue.first;
   x=(int) curElem->x-3;
   y=(int) curElem->y-11;
   curElem = curElem->next;
   x1=(int) curElem->x-11;
   y1=(int) curElem->y-8;


   EGLgetBlock (tempbgr,x,y,(x+42)>319?319:(x+42),y+34);
   EGLgetBlock (tempbgr1,x1,y1,(x1+31)>319?319:(x1+31),y1+34);

   for (int i=0;i<30;i++) {
      EGLputBlock (tempbgr,x,y);
      EGLputBlock (tempbgr1,x1,y1);
      temp.XSize = 43; temp.YSize = 34;
      temp.Image=diss_r_bmp->Image+2752*mov_frame;
      EGLputBlockT (&temp,x,y,1);
      temp.XSize = 31; temp.YSize = 34;
      temp.Image=diss_g_bmp->Image+1984*mov_frame;
      EGLputBlockT (&temp,x1,y1,1);
      EGLwaitVRetrace ();
      EGLwaitVRetrace ();
      EGLdrawVBuffer (screen1);

      mov_frame++;
   }

   AlpLim=tempAlp;

   //EGLfreeBlock (temp);
   EGLfreeBlock (tempbgr);
   EGLfreeBlock (tempbgr1);
}
#endif

/****************************************************************************\
* Function : AddObjects                                                      *
* Desc     : Adds players and npc ball =)s                                   *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void AddObjects (void)
{

   // Red ball properties
   TempElem.image  = r_ball;
   TempElem.speed  = 0;
   TempElem.xspeed = 0.5;
   TempElem.yspeed = 0.5;
   TempElem.x      = 80;
   TempElem.y      = 90;
   TempElem.weight = 1;
   TempElem.left   = RedKeys[0];
   TempElem.right  = RedKeys[1];
   TempElem.up     = RedKeys[2];
   TempElem.down   = RedKeys[3];
   TempElem.mode   = hitMain;

   AddObjToQueue (TempElem);

   // Green ball properties
   TempElem.image  = g_ball;
   TempElem.speed  = 0;
   TempElem.xspeed = -0.5;
   TempElem.yspeed = 0.5;
   TempElem.x      = 240;
   TempElem.y      = 90;
   TempElem.weight = 1;
   TempElem.left   = GreenKeys[0];
   TempElem.right  = GreenKeys[1];
   TempElem.up     = GreenKeys[2];
   TempElem.down   = GreenKeys[3];
   TempElem.mode   = hitMain;

   AddObjToQueue (TempElem);

   // White ball properties
   TempElem.image  = w_ball;
   TempElem.speed  = 0;
   TempElem.xspeed = 0.5-((float)random (100))/100;
   TempElem.yspeed = 0.5-((float)random (100))/100;
   TempElem.x      = 160;
   TempElem.y      = 90;
   TempElem.weight = 1;
   TempElem.left   = -1;
   TempElem.right  = -1;
   TempElem.up     = -1;
   TempElem.down   = -1;
   TempElem.mode   = hitAll;

   AddObjToQueue (TempElem);
}

/****************************************************************************\
* Function : Intro                                                           *
* Desc     : Litil intro before play                                         *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void Intro (void)
{
   int x,y,start=0,curpic=1;
   int exiti=0;
   int keysum=0, keycur=0;
   int lost=0;

   uchar *temp_sc=(uchar *) malloc (MaxX*MaxY*2);
   memset (temp_sc,0,MaxX*MaxY*2);

   for (y=0; y<GameMaxY; y++) {
      for (x=0; x<GameMaxX*2; x++) {
         temp_sc[y*MaxX*2+x] = logo_bmp->Image [y*GameMaxX*2+x];
      }
   }

   EGLclearVBuffer (screen1);
   EGLdrawVBuffer (screen1);

   // Draw logo !
   for (start=1; start<80; start++) {
      for (int len=0;len<32;len++) {
         // Up-Down
         for (x=0;x<MaxX;x++) {
            // +1 -1 !!!!
            if (len+start<83) {
               int p=(len+start)*MaxX+x;
               RedS   (screen1, p, RedR   (temp_sc,p,len));
               GreenS (screen1, p, GreenR (temp_sc,p,len << (GBits-5)));
               BlueS  (screen1, p, BlueR  (temp_sc,p,len));
            }
         }

         // Left to right
         for (y=94; y<110; y++) {
            // +1 -1 !!!!
            for (int i=0, p=len+start*4+y*MaxX;
                 i<4 && p<GameMaxX+y*MaxX;
                 i++, p++)
               GreenS (screen1, p, GreenR (temp_sc,p,len));
         }

         // Right to left
         for (y=83; y<94; y++) {
            // +1 -1 !!!!
            for (int i=0, p=GameMaxX-1-(len+start*4)+y*MaxX;
                 i<4 && p>=y*MaxX;
                 i++, p--) {
               RedS   (screen1, p, RedR   (temp_sc,p,len));
               GreenS (screen1, p, GreenR (temp_sc,p,len << (GBits-5)));
               BlueS  (screen1, p, BlueR  (temp_sc,p,len));
            }
         }
      }

      ProcessMessages();
      HRhan ();
      if (keymap [SDLK_ESCAPE] == PRESSED) {
         exiti=1;
         lost=1;
         break;
      }
      for (int i=0; i<512; i++)
         if (keymap [i] == PRESSED) {
            exiti=1;
            break;
         }
      EGLwaitVRetrace ();
      delay (8);
      EGLdrawVBuffer (screen1);
      if (exiti) break;
   }

   for (int i=0; i<512; i++)
      keysum+=keymap[i];

   while (!exiti) {
      // Read data for next frame
      for (y=105; y<GameMaxY; y++) {
         for (x=0; x<GameMaxX*2; x++) {
            temp_sc[y*MaxX*2+x]=cpic_bmp->Image [(y-105)*GameMaxX*2+x+(curpic-1)*GameMaxX*2*95];
         }
      }
      curpic++;
      if (curpic>5) curpic=1;

      // Appears from bottom 
      for (start=GameMaxY-1; start>110-32; start--) {
         for (int len=0; len<32; len++) {
            for (x=0; x<GameMaxX; x++) {
               // +1 -1 !!!!
               int p=(start+len)*MaxX + x;
               if (start+len>110 && start+len<MaxY) {
                  RedS   (screen1, p, RedR   (temp_sc, p, 32-len));
                  GreenS (screen1, p, GreenR (temp_sc, p, (32-len) << (GBits-5)));
                  BlueS  (screen1, p, BlueR  (temp_sc, p, 32-len));
               }
            }
         }
         ProcessMessages();
         EGLwaitVRetrace ();
         delay (8);
         EGLdrawVBuffer (screen1);
         keycur=0;
         HRhan ();
         if (KeySum() != 0) {exiti=1;keysum=255;break;}
      }

      // Wait 5 seconds
      for (y=0;y<400;y++) {
         ProcessMessages();
         HRhan ();
         if (KeySum() != 0) {exiti=1;keysum=255;break;}
         delay (13);
      }

      // Read date for next frame
      for (y=105; y<GameMaxY; y++) {
         for (x=0; x<GameMaxX*2; x++) {
            temp_sc[y*MaxX*2+x] = cpic_bmp->Image [(y-105)*GameMaxX*2+x+(curpic-1)*GameMaxX*2*95];
         }
      }
      curpic++;
      if (curpic>5) curpic=1;

      for (start=103-32; start<GameMaxY; start++) {
         for (int len=0;len<32;len++) {
            for (x=0; x<GameMaxX; x++) {
               // +1 -1 !!!!
               int p = (start+len)*MaxX + x;
               if (start+len>110 && start+len<MaxY) {
                  RedS   (screen1, p, RedR   (temp_sc, p, len));
                  GreenS (screen1, p, GreenR (temp_sc, p, len << (GBits-5)));
                  BlueS  (screen1, p, BlueR  (temp_sc, p, len));
               }
            }
         }
         ProcessMessages();
         EGLwaitVRetrace ();
         delay (8);
         EGLdrawVBuffer (screen1);
         HRhan ();
         if (KeySum() != 0) {
            exiti=1;
            keysum=255;
            break;
         }

      }

      // Wait 5 seconds
      for (y=0;y<400;y++) {
         ProcessMessages();
         HRhan ();
         if (KeySum() != 0) {
            exiti=1;
            keysum=255;
            break;
         }
         delay (13);
      }
      if (keymap[SDLK_ESCAPE]) lost=1;

   }

   ProcessMessages();
   EGLwaitVRetrace ();
   EGLdrawVBuffer (screen1);

   if (NOSOUND==NO)
	   ElPlaySample (SAMPLE_SELECT, 3000);

   free (temp_sc);

   if (lost) {
      DeinitKbd();
      ElSoccDone();
   }

   EGLdrawVBuffer (screen1);
}


/****************************************************************************\
* Function : RedS                                                            *
* Desc     : Update red component of pixel                                   *
* Input    : pixel, new value                                                *
* Output   : None                                                            *
\****************************************************************************/
void RedS (unsigned char * pix, int addr, unsigned short value) {
   unsigned short temp;

   temp = ((unsigned short *) pix)[addr];

   temp = (temp & (GMask | BMask)) | value;

   ((unsigned short *)pix)[addr] = temp;
}

/****************************************************************************\
* Function : RedR                                                            *
* Desc     : Get red component of pixel                                      *
* Input    : pixel                                                           *
* Output   : value                                                           *
\****************************************************************************/
unsigned short RedR (unsigned char * pix,int addr,int len) {
   ushort temp;

   temp = ((unsigned short *)pix)[addr];

   temp=(temp & RMask) >> RShift;

   if (temp > len) temp-=len;
   else temp=0;

   temp=(temp << RShift) & RMask;

   return temp;
}

/****************************************************************************\
* Function : GreenS                                                          *
* Desc     : Update green component of pixel                                 *
* Input    : pixel, new value                                                *
* Output   : None                                                            *
\****************************************************************************/
void GreenS (uchar * pix, int addr, unsigned short value) {
   ushort temp;

   temp = ((ushort *) pix)[addr];

   temp = (temp & (RMask | BMask)) | value;

   ((ushort *)pix)[addr] = temp;
}

/****************************************************************************\
* Function : GreenR                                                          *
* Desc     : Read green component of pixel                                   *
* Input    : pixel                                                           *
* Output   : value                                                           *
\****************************************************************************/
unsigned short GreenR (uchar * pix,int addr,int len) {
   ushort temp = ((ushort *)pix)[addr];

   temp = (temp & GMask) >> GShift;

   if (temp > len) temp-=len;
   else temp=0;

   temp = (temp << GShift) & GMask;

   return (temp);
}

/****************************************************************************\
* Function : BlueS                                                           *
* Desc     : Update blue component of pixel                                  *
* Input    : pixel, new value                                                *
* Output   : None                                                            *
\****************************************************************************/
void BlueS (uchar * pix, int addr, unsigned short value) {
   ushort temp;

   temp = ((ushort *) pix)[addr];

   temp = (temp & (RMask | GMask)) | value;

   ((ushort *)pix)[addr] = temp;
}

/****************************************************************************\
* Function : BlueR                                                           *
* Desc     : Read blue component of pixel                                    *
* Input    : pixel                                                           *
* Output   : value                                                           *
\****************************************************************************/
unsigned short BlueR (uchar * pix,int addr,int len) {
   ushort temp;

   temp = ((ushort *)pix)[addr];

   temp = (temp & BMask) >> BShift;

   if (temp > len) temp-=len;
   else temp=0;

   temp = (temp << BShift) & BMask;

   return (temp);
}



/****************************************************************************\
* Function : EGLputBlockT                                                    *
* Desc     : Puts rectangle region of picture in EGLvBuffer                  *
* Input    : Source bitmap, x ,y, transparent color                          *
* Output   : EGLok/EGLerror                                                  *
\****************************************************************************/
int EGLputBlockT (BitMap* block,
                  short x,
                  short y,
                  short tc) {

   if (block == NULL) return EGLerror;
   tc=tc;

   ushort xrl,yrl;
   ushort xc,yc;
   unsigned short *p, *destp;

   if (y+block->YSize > MaxY) yrl=(ushort) (MaxY-1-y);
   else yrl = block->YSize-1;
   if (x+block->XSize > MaxX) xrl=(ushort) (MaxX-1-x);
   else xrl = block->XSize-1;

   p=(unsigned short *) block->Image;

   for (yc=y; yc<=yrl+y; yc++) {
      destp=(unsigned short*) EGLvBuffer+(yc*MaxX+x);
      for (xc=x; xc<=(xrl+x); xc++) {

         ushort r=(ushort) ((*p >> RShift) & RMaskShifted) << 1;
         ushort g=(ushort) ((*p >> GShift) & GMaskShifted) << (6-GBits);
         ushort b=(ushort) ((*p >> BShift) & BMaskShifted) << 1;

         if (r > AlpLim || g > AlpLim || b > AlpLim)
            *destp++ = *p++;

          else {
             float maxComp=(float) max3 (r, g, b);

             transA=(maxComp+corr*20)/64;
             if (transA>1) transA=1;
             transB=1-transA;

             *destp++ = ((int)((*p & RMask)*transA) + (int)((*destp & RMask)*transB)) & RMask |
                        ((int)((*p & GMask)*transA) + (int)((*destp & GMask)*transB)) & GMask |
                        ((int)((*p & BMask)*transA) + (int)((*destp & BMask)*transB)) & BMask;

             p++;
          }
      }
      if (x+block->XSize > MaxX)
         p += block->XSize-xrl-1;

   }

   return EGLok;
}

/****************************************************************************\
* Function : ScaleToVBuf                                                     *
* Desc     : Scales image to virtual screen                                  *
* Input    : source image, new size, position and Vbuf adress                *
* Output   : None                                                            *
\****************************************************************************/
void ScaleToVBuf (BitMap * source,
                  int newXsize,
                  int newYsize,
                  int x,
                  int y,
                  unsigned char * destination) {

   double coofx,
          coofy,
          tx,
          ty;

   unsigned char * sou;
   char byte;

   short xx,yy;

   sou=source->Image;

   coofx = source->XSize / (double)(newXsize+1);
   coofy = source->YSize / (double)(newYsize+1);

   ty=0;
   for (yy=0;yy<newYsize;yy++) {
         ty=ty+coofy;
         tx=0;
      for (xx=0;xx<newXsize;xx++) {
         tx=tx+coofx;
         byte=sou[((int)ty)*(source->XSize)*2+((int)tx)*2];
         if (byte!=0) destination [(yy+y)*MaxX*2+xx*2+x*2]=byte;
         byte=sou[(int)((int) ty)*(source->XSize)*2+((int) tx)*2+1];
         if (byte!=0) destination [(yy+y)*MaxX*2+xx*2+x*2+1]=byte;
      }
   }
}

/****************************************************************************\
* Function : scaleOne                                                        *
* Desc     : Scales one object from nothing to full size                     *
* Input    : Object to scal and where to scale                               *
* Output   : None                                                            *
\****************************************************************************/
void scaleOne (BitMap * source, int xpl, int ypl, int localGoal)
{

   int xsize;
   BitMap * temp1;

   EGLallocBlock (&temp1, source->XSize+1,source->YSize+1);

      EGLgetBlock (temp1,
                   xpl,
                   ypl-(source->YSize+1)/2,
                   xpl+source->XSize+1,ypl+(source->YSize+1)/2);


   for (xsize=0; xsize<source->XSize+1; xsize+=3) {
      EGLputBlock (temp1,xpl,ypl-(source->YSize+1)/2);
      EGLgetBlock (temp1,
                   xpl,
                   ypl-(source->YSize+1)/2,
                   xpl+source->XSize+1,ypl+(source->YSize+1)/2);
      ScaleToVBuf (source,xsize,
                   (int) ((float)xsize/(float)(source->XSize+1)*(source->YSize+1)),
                   xpl+((source->XSize+1)-xsize)/2,
                   (int) (ypl-((float)xsize/(float)(source->XSize+1)*(source->YSize+1))/2),
                   screen1);
      EGLwaitVRetrace ();
      EGLdrawVBuffer (screen1);
   }

   if (localGoal) {
      EGLputBlock (temp1,xpl,ypl-(source->YSize+1)/2);
      for (int e=0;e<30;e++) EGLwaitVRetrace ();
   }

   EGLfreeBlock (temp1);
}

/****************************************************************************\
* Function : DrawPlasma                                                      *
* Desc     : Generates plasma.                                               *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void DrawPlasma() {
  uchar i2, j2, c, yp;
  unsigned char *buf_ptr;
  int x,y;

  // EGLtrace ("i3=%i, j3=%i\n", i3, j3);
  i3--; j3++;
  if (end_y < 200) end_y+=2;

  buf_ptr=plasma256;
  /* sin and cos are freely interchangeable */
  for (y=0; y<end_y; y++) {
    i2=SinTable [(y +i3 ) & 0xFF];
    j2=SinTable [(uchar) j3];
    yp=CosTable [(y+j2) & 0xFF];
    for (x=0; x<PlasmaX+2; x++) {
      c=CosTable [(x+i2) & 0xFF] + yp;
      *buf_ptr++ = c;
    }
  }
}

/****************************************************************************\
* Function : ConvertPlasma                                                   *
* Desc     : Converts plasma from indexed color to high color and            *
* Desc     : draws it at center of game screen.                              *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void ConvertPlasma (void) {
   ushort *f_char;
   unsigned char * f_pl;

   f_char=(ushort *) screen1;
   f_pl=plasma256;

   f_char+=16*MaxX;
   f_char=f_char+8;
   for (int y=16; y<PlasmaY+18; y++) {
      for (int x=0; x<PlasmaX+2; x++) {
         *f_char++ = Ctab[*f_pl++];
      }
      f_char += MaxX-PlasmaX-2;
   }
}

/****************************************************************************\
* Function : SlideIn                                                         *
* Desc     : Slides in the specified image                                   *
* Input    : Image, image width                                              *
* Output   : None                                                            *
\****************************************************************************/

void SlideIn (BitMap *bitMap, int height)
{
   int BarSize = MaxX*6*2;
   char *gr_ux=(char*) malloc (MaxX*10*2);
   char *gr_dx=(char*) malloc (MaxX*10*2);
   memcpy (gr_ux, screen1+MaxX*2*94, BarSize);
   memcpy (gr_dx, screen1+MaxX*2*100, BarSize);

   for (int dy=0; dy<height; dy++) {
      memcpy (screen1 + (94-dy) * (MaxX*2), gr_ux, BarSize);

      memcpy (screen1 + (100-dy) * (MaxX*2), bitMap->Image + (height-dy)*(GameMaxX*2), GameMaxX*2);
      memcpy (screen1 + (99+dy) * (MaxX*2),  bitMap->Image + (height-1+dy)*(GameMaxX*2), GameMaxX*2);

      memcpy (screen1 + (100+dy) * (MaxX*2), gr_dx, BarSize);

      ProcessMessages();
      EGLwaitVRetrace ();
      EGLdrawVBuffer (screen1);
   }

   free (gr_ux);
   free (gr_dx);
}

/****************************************************************************\
* Function : GrHelp                                                          *
* Desc     : OnLine help                                                     *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void GrHelp (void) {
   BitMap *gr_help;

   if  (inRed==YES) return;
   inRed=YES;

   char *old_scr=(char*) malloc (MaxX*MaxY*2);

   memcpy (old_scr, screen1, MaxX*MaxY*2);

   gr_help=EGLloadFFI (ID_HELP);

   int w;
   for (w=0; w<GameMaxX-4; w+=5) {

      EGLputBlock (gr_u,w,94);
      EGLputBlock (gr_d,GameMaxX-5-w,100);
      /*if (!(w%8)) {*/
         EGLwaitVRetrace ();
         EGLdrawVBuffer (screen1);
      /*}*/
   }

   SlideIn (gr_help, 91);
   Getch ();
   WaitRelease();

//   MorphScreen (old_scr,screen1);

   memcpy (screen1, old_scr, MaxX*MaxY*2);

   free (old_scr);
   free (gr_help);
   ResUnload (ID_HELP);

   inRed=NO;

   EGLdrawVBuffer (screen1);
}

/****************************************************************************\
* Function : RedefineKeys                                                    *
* Desc     : Redefinition of player keys                                     *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void RedefineKeys (void) {
   BitMap *redef;

   if  (inRed==YES) return;
   inRed=YES;

   unsigned char *old_scr=(unsigned char*) malloc (MaxX*MaxY*2);

   redef=EGLloadFFI (ID_REDEFINE);
   gr_ok=EGLloadFFI (ID_OK);
   gr_quest=EGLloadFFI (ID_QUEST);

   memcpy (old_scr, screen1, MaxX*MaxY*2);

   for (int w=0; w<GameMaxX-4; w+=5) {

      EGLputBlock (gr_u,w,94);
      EGLputBlock (gr_d,GameMaxX-5-w,100);
      /*if (!(w%8)) {*/
         EGLwaitVRetrace ();
         EGLdrawVBuffer (screen1);
      /*}*/
   }

   SlideIn (redef, 80);

   RedKeys[0]=getKey (90,93);
   RedKeys[1]=getKey (90,113);
   RedKeys[2]=getKey (90,135);
   RedKeys[3]=getKey (90,157);

   GreenKeys[0]=getKey (205,93);
   GreenKeys[1]=getKey (205,113);
   GreenKeys[2]=getKey (205,135);
   GreenKeys[3]=getKey (205,157);

   if (isQueue==YES) {
      Qelem * curElem;

      curElem = queue.first;
      curElem->left  = RedKeys[0];
      curElem->right = RedKeys[1];
      curElem->up    = RedKeys[2];
      curElem->down  = RedKeys[3];

      curElem = curElem->next;

      curElem->left  = GreenKeys[0];
      curElem->right = GreenKeys[1];
      curElem->up    = GreenKeys[2];
      curElem->down  = GreenKeys[3];
   }

   MorphScreen (old_scr,screen1);

   memcpy (screen1, old_scr, MaxX*MaxY*2);

   free (old_scr);

   free (redef);
   ResUnload (ID_REDEFINE);
   free (gr_ok);
   ResUnload (ID_OK);
   free (gr_quest);
   ResUnload (ID_QUEST);

   // Save keys. In fact this adds redefined keys to the end of the ini file.
   // But i'll let on that this is as intended :)
   {   
      FILE * ini;  
      ini=fopen ("elsocc.ini", "at");
      if (ini!=NULL) {
         fprintf (ini, "\n\
   ; Redefined keys\n\
   [Controls/RedPlayer]\n\
       Left=%d\n\
       Right=%d\n\
       Up=%d\n\
       Down=%d\n\
\n\
   [Controls/GreenPlayer]\n\
       Left=%d\n\
       Right=%d\n\
       Up=%d\n\
       Down=%d\n", 
       RedKeys[0],RedKeys[1],RedKeys[2],RedKeys[3],
       GreenKeys[0],GreenKeys[1],GreenKeys[2],GreenKeys[3]);

         fclose (ini); 
      }
   }

   inRed=NO;

   EGLdrawVBuffer (screen1);
}

/****************************************************************************\
* Function : getKey                                                          *
* Desc     : gets one keystroke                                              *
* Input    : None                                                            *
* Output   : key scanned                                                     *
\****************************************************************************/
int getKey (int x, int y) {
   char keyTemp [512];
   int jj;

   EGLputBlock (gr_quest,x,y);
   EGLdrawVBuffer (screen1);

   memcpy (keyTemp, keymap, 512);

   do {
      ProcessMessages();
      jj=0;
      for (int j=0;j<512;j++) {
         if (keymap[j]!=keyTemp[j]) jj=j;
      }
   } while (!jj);

   EGLputBlock (gr_ok,x,y);
   EGLdrawVBuffer (screen1);

   // Wait until all keys are depressed
   WaitRelease();

   return (jj);
}


/****************************************************************************\
* Function : MorphScreen                                                     *
* Desc     : Morphs one screen into another                                  *
* Input    : src & destination                                               *
* Output   : None                                                            *
\****************************************************************************/
void MorphScreen (unsigned char * source, unsigned char * destination) {

   for (int count=0;count<32;count++) {
      for (int y=0; y<GameMaxY; y++) {
         for (int x=0; x<GameMaxX; x++) {

           unsigned short SrcC, DestC;
//           if (!(count%2)) {
              SrcC=RedR(source,x+y*MaxX,0);
              DestC=RedR(destination,x+y*MaxX,0);

              if (SrcC > DestC)
                  RedS (destination, x+y*MaxX, DestC+(1 << RShift));
              if (SrcC < DestC)
                  RedS (destination, x+y*MaxX, DestC-(1 << RShift));
//           }

           SrcC = GreenR(source, x+y*MaxX, 0);
           DestC = GreenR(destination, x+y*MaxX, 0);
           if (SrcC > DestC)
               GreenS(destination, x+y*MaxX, DestC+(1 << GShift));
           if (SrcC < DestC)
               GreenS(destination, x+y*MaxX, DestC-(1 << GShift));

//           if (!(count%2)) {
              if (BlueR(source, x+y*MaxX,0) > BlueR(destination, x+y*MaxX, 0))
                  BlueS(destination, x+y*MaxX, BlueR(destination, x+y*MaxX, 0)+(1 << BShift));
              if (BlueR(source, x+y*MaxX,0) < BlueR(destination, x+y*MaxX, 0))
                  BlueS(destination, x+y*MaxX, BlueR(destination, x+y*MaxX, 0)-(1 << BShift));
//           }
         }
      }
      EGLwaitVRetrace ();
      EGLdrawVBuffer (destination);
   }
}

/****************************************************************************\
* Function : Demix                                                           *
* Desc     : Restores and writes one file to disk                            *
* Input    : resource handle and file name                                   *
* Output   : None                                                            *
\****************************************************************************/
void Demix (HRESELEM handle, char * name)
{
   long CharsDone = 0;

   long ResSize = ResGetSize (handle);
   uchar *ResBuf = ResLoad (handle);
   FILE *f = fopen (name, "wb");

   char x = 0;

   int k = ResBuf [CharsDone];

   do {
      k += x;
      k ^= x;
      k -= 3;
      putc (k, f);
      CharsDone++;
      k = ResBuf [CharsDone];
      x += 3;
   } while (CharsDone<ResSize);

   ResUnload (handle);
   fclose (f);
}
