/****************************************************************************\
*  *
* File 'el_socc.h'    - definitions for Electron Soccer                      *
* Project started     - 14:54:21, 18-11-97                                   *
* Last modified       - 19:33:44,  1-12-97                                   *
*                                                                            *
* Written by EXeLENZ (2:5030/448.1@fidonet, ivan@solvo.ru)                   *
*  *
\****************************************************************************/

#include "./egl_sdl/egl.h"
#include "./egl_sdl/egl_serv.h"
#include "./egl_sdl/egl_defs.h"

#define hitAll  1
#define hitMain 2
#define mess    3

#define YES     1
#define NO      0

#define PRESSED   1
#define DEPRESSED 0

#define Lower     1
#define Upper     2

#define VorX      136
#define VorSize   48


#define DISSOLVE_BALLS

const double rad=57.295779513;        

//  । 쥪⮢
struct Qelem {

   int Id;                             /* Object id                         */

   BitMap * image;                     /* Object image                      */
   BitMap * bgr;                       /* Object background                 */

   float speed;                        /* Object speed     90             */
   double xspeed;                      /* x and y         y                */
   double yspeed;                      /* speed            x0 */
   double x;                           /*  coordinate (upper left corner)  */
   double y;                           /* Y coordinate                      */

   double ox;                          /*  coordinate (lower left corner)  */
   double oy;                          /* Y coordinate (on previous step )  */

   short weight;                       /* Object weight                     */

   Qelem * LastCollide;                /* Last collision                    */

   short left;                         /* Object control (left)             */
   short right;                        /* Object control (right)            */
   short up;                           /* Object control (up)               */
   short down;                         /* Object control (down)             */

   char mode;                          /* hitAll  - Can be hit by all       */
                                       /* hitMain - Can be hit only by      */
                                       /*           hitAll type objects     */
                                       /* mess    - object type message     */

   Qelem * next;                       /* Next object                       */
   Qelem * prev;                       /* Previous object                  */
};

extern char keymap [512];
extern int NOSOUND, NOPLASMA, NOMUSIC;
extern VBuffer screen1;

/* Virtual screen size */
enum { MaxX=320, MaxY=200 };
/* Playfield size, determined by resolution */
enum { GameMaxX=320, GameMaxY=200 };
/* Background plasma size */
enum { PlasmaX=302, PlasmaY=166 };
/* Physical videomode resolution */
extern int VidMaxX, VidMaxY;

#ifdef USE_MIDAS
extern int m_error;
#endif

extern double GameSpeed;
extern int GameTime;

void Intro (void);
void Game (void);

void KbdHandler (void);
void ControlObject (Qelem * elem);

void InitObjQueue (void);
void DestroyQueue (void);

void AddObjToQueue (Qelem TempElem);

/****************************************************************************\
* Function : ProcessQueue                                                    *
* Desc     : Manages all objects in queue (moves them, etz)                  *
* Input    : None                                                            *
* Output   : Goal status                                                     *
\****************************************************************************/
int ProcessQueue (long FrameElapsed);

void WriteScores (void);

/****************************************************************************\
* Function : WriteTime                                                       *
* Desc     : Draws time left                                                 *
* Input    : Time left                                                       *
* Output   : None                                                            *
\****************************************************************************/
void WriteTime (int timeLeft);

void AddObjects (void);

void BounceWalls   (Qelem * curElem, double move_x, double move_y);
void BounceObjects (Qelem * curElem);

/****************************************************************************\
* Function : CheckGoal                                                       *
* Desc     : Checks for winning conditions                                   *
* Input    : Object to handle                                                *
* Output   : Goal status                                                     *
\****************************************************************************/
int CheckGoal (Qelem * curElem);

void greenWin (void);
void redWin (void);
void timeOver (void);

void dissolveBalls (void);

int EGLputBlockT (BitMap* block,
                  short x,
                  short y,
                  short tc);

void ScaleToVBuf (BitMap * source,
                  int newXsize,
                  int newYsize,
                  int x,
                  int y,
                  char * destination);

/****************************************************************************\
* Function : scaleOne                                                        *
* Desc     : Scales one object from nothing to full size                     *
* Input    : Object to scal and where to scale                               *
* Output   : None                                                            *
\****************************************************************************/
void scaleOne (BitMap * source, int xpl, int ypl, int localGoal=0);

/****************************************************************************\
* Function : RedS                                                            *
* Desc     : Update red component of pixel                                   *
* Input    : pixel, new value                                                *
* Output   : None                                                            *
\****************************************************************************/
void RedS (unsigned char * pix, int addr, unsigned short value);

/****************************************************************************\
* Function : RedR                                                            *
* Desc     : Get red component of pixel                                      *
* Input    : pixel                                                           *
* Output   : value                                                           *
\****************************************************************************/
unsigned short RedR (unsigned char * pix,int addr,int len);

/****************************************************************************\
* Function : GreenS                                                          *
* Desc     : Update green component of pixel                                 *
* Input    : pixel, new value                                                *
* Output   : None                                                            *
\****************************************************************************/
void GreenS (unsigned char * pix, int addr, unsigned short value);

/****************************************************************************\
* Function : GreenR                                                          *
* Desc     : Read green component of pixel                                   *
* Input    : pixel                                                           *
* Output   : value                                                           *
\****************************************************************************/
unsigned short GreenR (unsigned char * pix,int addr,int len);

/****************************************************************************\
* Function : BlueS                                                           *
* Desc     : Update blue component of pixel                                  *
* Input    : pixel, new value                                                *
* Output   : None                                                            *
\****************************************************************************/
void BlueS (unsigned char * pix, int addr, unsigned short value);

/****************************************************************************\
* Function : BlueR                                                           *
* Desc     : Read blue component of pixel                                    *
* Input    : pixel                                                           *
* Output   : value                                                           *
\****************************************************************************/
unsigned short BlueR (unsigned char * pix,int addr,int len);

int max3 (int a,
          int b,
          int c);

void Getch (void);

/****************************************************************************\
* Function : WaitRelease                                                     *
* Desc     : Waits till all keys are released                                *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void WaitRelease (void);

void Error (int cause);

/****************************************************************************\
* Function : DrawPlasma                                                      *
* Desc     : Generates plasma.                                               *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void DrawPlasma();

/****************************************************************************\
* Function : ConvertPlasma                                                   *
* Desc     : Converts plasma from indexed color to high color and            *
* Desc     : draws it at center of game screen.                              *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void ConvertPlasma (void);

/****************************************************************************\
* Function : GrHelp                                                            *
* Desc     : OnLine help                                                     *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void GrHelp (void);

/****************************************************************************\
* Function : RedefineKeys                                                      *
* Desc     : Redefinition of player keys                                     *
* Input    : None                                                            *
* Output   : None                                                            *
\****************************************************************************/
void RedefineKeys (void);

/****************************************************************************\
* Function : HRhan                                                           *
* Desc     : Handles help and redefine                                       *
* Input    : Object                                                          *
* Output   : None                                                            *
\****************************************************************************/
void HRhan (void);

/****************************************************************************\
* Function : MorphScreen                                                       *
* Desc     : Morphs one screen into another                                  *
* Input    : src & destination                                               *
* Output   : None                                                            *
\****************************************************************************/
void MorphScreen (unsigned char * source, unsigned char * destination);

/****************************************************************************\
* Function : getKey                                                          *
* Desc     : gets one keystroke                                              *
* Input    : None                                                            *
* Output   : key scanned                                                     *
\****************************************************************************/
int getKey (int x, int y);

/****************************************************************************\
* Function : Demix                                                           *
* Desc     : Restores and writes one file to sdisk                           *
* Input    : resource handle and file name                                   *
* Output   : None                                                            *
\****************************************************************************/
void Demix (int handle, char * name);

BitMap * EGLloadFFI (int handle);

/****************************************************************************\
* Function : EGLtrace                                                        *
* Desc     : ⫠筠                                                *
* Input    : ଠ⭠ ப, 㬥                                     *
* Output   :                                                              *
\****************************************************************************/
void /*__cdecl*/ EGLtrace (char *fmt, ...);

/* System independant main */
void ElSoccMain();
void UpdatePixelFormat();
void UpdateBuffer (int OldMaxX, int OldMaxY);
void ElSoccDone();

void ProcessMessages();
void DeinitKbd();

extern "C" void delay (unsigned int __milliseconds);
