/* $Header: /C/FaerieX/fxRes.cpp 2     20.07.98 22:29 Yole $
   FaerieX: resource manager implementation file
   (C) 1998-2001 Dmitry Jemerov (2:5030/654, yole@spb.cityline.ru)
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <zlib.h>
#include "fxres.h"

struct ResElem
{
  ResDirEntry de;
  unsigned char *mem;
  int locks;
};


static FILE *resf;
static ResHeader rh;
static ResElem *ResDir;
static int Compressed = 0;

//
// ResLoadDir()
// Loads resource directory

static int ResLoadDir()
{
  fread (&rh, sizeof (ResHeader), 1, resf);
  if (!strncmp (rh.Sign, "FaeR", 4))
    Compressed = 0;
  else if (!strncmp (rh.Sign, "FaeZ", 4))
    Compressed = 1;
  else
    return 0;

  if (fseek (resf, rh.DirOffset, SEEK_SET)) return 0;
  ResDir = new ResElem [rh.ResCount];
  if (!ResDir) return 0;
  for (int i=0; i<rh.ResCount; i++)
  {
    fread (&ResDir [i].de, sizeof (ResDirEntry), 1, resf);
    ResDir [i].mem = NULL;
    ResDir [i].locks = 0;
  }
  return 1;
}

//
// ResInitFile()
// Initializes resource file fname

int ResInitFile (const char *fname)
{
  if (resf || ResDir) ResClose();
  resf=fopen (fname, "rb");
  if (!resf)
  {
    //TRACE ("Cannot open resource file %s\n", fname);
    return 0;
  }
  return ResLoadDir();
}

//
// ResInitBound()
// Initializes resource bound to file

int ResInitBound (const char *fname)
{
  if (resf || ResDir) ResClose();
  resf=fopen (fname, "rb");
  if (!resf) return 0;
  fseek (resf, -4, SEEK_END);
  long BindPos;
  fread (&BindPos, 4, 1, resf);
  fseek (resf, BindPos, SEEK_SET);
  return ResLoadDir();
}

//
// ResClose()
// Closes resource file, frees memory

void ResClose()
{
  if (resf)
  {
    fclose (resf);
    resf = NULL;
  }
  if (ResDir)
  {
    for (int i=0; i<rh.ResCount; i++)
      if (ResDir [i].mem) delete ResDir [i].mem;
    delete ResDir; ResDir = NULL;
  }
}

//
// ResLoad()
// Loads resource element, returns pointer or NULL if failed

unsigned char *ResLoad (HRESELEM hRes)
{
  if (!ResDir) return NULL;
  int ResID = (int) hRes;
  if (ResID > rh.ResCount) return NULL;
  ResElem &re=ResDir [ResID];
  if (re.mem)
  {
    re.locks++;
    return re.mem;
  }
  if ((re.de.Type >> 8 == 0xFF) || (re.de.RealSize == 0))
    re.de.RealSize = ResGetSize (ResID);

  re.mem = new unsigned char [re.de.RealSize];
  if (!re.mem) return NULL;
  if (re.de.Type >> 8 == 0xFF)
  {
    char *Path=new char [re.de.PackedSize];
    if (fseek (resf, re.de.Offset, SEEK_SET)) return NULL;
    fread (Path, re.de.PackedSize, 1, resf);
    FILE *ref=fopen (Path, "rb");
    if (!ref) return NULL;
    fread (re.mem, re.de.RealSize, 1, ref);
    fclose (ref);
    delete Path;
  }
  else {
    if (fseek (resf, re.de.Offset, SEEK_SET)) return NULL;
    if (Compressed)
    {
      unsigned char *PackBuf = new unsigned char [re.de.PackedSize];
      fread (PackBuf, re.de.PackedSize, 1, resf);
      unsigned long unpackedSize = re.de.RealSize;
      uncompress (re.mem, &unpackedSize, PackBuf, re.de.PackedSize);
      delete PackBuf;
    }
    else
      fread (re.mem, re.de.PackedSize, 1, resf);
  }
  re.locks=1;
  return re.mem;
}

//
// ResUnload()
// Unloads a resource element, frees memory if no more locks

void ResUnload (HRESELEM hRes)
{
  if (!ResDir) return;
  int ResID = (int) hRes;
  if (ResID > rh.ResCount) return;
  ResElem &re=ResDir [ResID];
  re.locks--;
  if (re.locks == 0)
  {
    delete re.mem;
    re.mem = NULL;
  }
}

//
// ResGetSize()
// Gets unpacked size of a resource element

long ResGetSize (HRESELEM hRes)
{
  if (!ResDir) return -1;
  int ResID = (int) hRes;
  if (ResID > rh.ResCount) return -1;
  ResElem &re=ResDir [ResID];

  if ((re.de.Type >> 8 == 0xFF) && (re.de.RealSize == 0))
  {
    char *Path=new char [re.de.PackedSize];
    if (fseek (resf, re.de.Offset, SEEK_SET)) return -1;
    fread (Path, re.de.PackedSize, 1, resf);
    FILE *ref=fopen (Path, "rb");
    if (!ref) return -1;
    fseek (ref, 0, SEEK_END);
    re.de.RealSize = ftell (ref);
    fclose (ref);
    delete Path;
  }
  return re.de.RealSize;
}
