/* $Header: /C/FaerieX/fxRes.h 1     20.07.98 22:33 Yole $
   FaerieX: resource manager header file
   (C) 1998-2001 Dmitry Jemerov (2:5030/654, yole@spb.cityline.ru)
*/

#ifndef __FAERES_H
#define __FAERES_H

typedef int HRESELEM;

// Resource header. Located in the beginning of resource file.
struct ResHeader
{
  char Sign [4]; //FaeR or FaeZ (for zlib-compressed resources)
  long DirOffset; //the resource directory is at the end of file
  long ResCount;
};

// Resource directory entry.
struct ResDirEntry
{
  long Offset;
  long RealSize;
  long PackedSize;
  unsigned short Type;  // can be used arbitrarily. Specifies compression?
  /* if (Type >> 8 == 0xFF) - this is an indirect element
     (the resource entry contains the string with absolute path to
     resource entry file, RealSize = 0, PackedSize = strlen (path). */
  unsigned short __padding;   // ensure 4-byte alignment
};

/****************************************************************************\
* Function : ResInitFile                                                     *
* Desc     : Initializes resource file fname                                 *
* Input    : resource file name                                              *
* Output   : 1 - success, 0 - failure                                        *
\****************************************************************************/
int ResInitFile (const char *fname);

/****************************************************************************\
* Function : ResInitBound                                                    *
* Desc     : Initializes resource bound to file                              *
* Input    : bound resource file name                                        *
* Output   : 1 - success, 0 - failure                                        *
\****************************************************************************/
int ResInitBound (const char *fname);

/****************************************************************************\
* Function : ResClose                                                        *
* Desc     : Close resource file, free memory                                *
* Input    : none                                                            *
* Output   : none                                                            *
\****************************************************************************/
void ResClose();

/****************************************************************************\
* Function : ResLoad                                                         *
* Desc     : Load resource element, return pointer                           *
* Input    : resource ID                                                     *
* Output   : pointer to memory; NULL if failed                               *
\****************************************************************************/
unsigned char *ResLoad (HRESELEM hRes);

/****************************************************************************\
* Function : ResUnload                                                       *
* Desc     : Unload resource element, free memory if no more locks           *
* Input    : resource ID                                                     *
* Output   : none                                                            *
\****************************************************************************/
void ResUnload (HRESELEM hRes);

/****************************************************************************\
* Function : ResGetSize                                                      *
* Desc     : Get unpacked size of resource element                           *
* Input    : resource ID                                                     *
* Output   : length or -1 if failed                                          *
\****************************************************************************/
long ResGetSize (HRESELEM hRes);

#endif